#include "stdafx.h"
#include "uusbb_winusb.h"


//WinUsb---------------------------------------------------------------------------------

bool CUUSBB_WinUsb::Open(const GUID &guid, unsigned short vId, unsigned short pId)
{
	if(!this->loaded)
		return false;

	BOOL bResult;
	USB_INTERFACE_DESCRIPTOR ifaceDescriptor;
	WINUSB_PIPE_INFORMATION pipeInfo;

	for(int i=0; (this->deviceHandle = OpenDevice(i, guid)) != NULL; i++)
	{
		bResult = ptrWinUsb_Initialize(this->deviceHandle, &(this->winUSBHandle));
		if(bResult)
			break;
		CloseHandle(this->deviceHandle);
	}

	if(this->deviceHandle == NULL)
	{
		MessageBox(NULL, _T("USBfoCX܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
		return false;
	}

	bResult = ptrWinUsb_QueryInterfaceSettings(this->winUSBHandle, 0, &ifaceDescriptor);

	if(bResult)
	{
		for(int i=0;i<ifaceDescriptor.bNumEndpoints;i++)
		{
			bResult = ptrWinUsb_QueryPipe(this->winUSBHandle, 0, (UCHAR) i, &pipeInfo);
			switch(pipeInfo.PipeId)
			{
			case 0x02:
			case 0x84:
			case 0x86:
				break;
			default:
				MessageBox(NULL, _T("USBfoCX̃pCvُłB\nԈfoCXWinUSBCXg[ꂽ\܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
				return false;
			}
			//Xg[͎ŃNA
			UCHAR acs = 1;
			ptrWinUsb_SetPipePolicy(this->winUSBHandle, pipeInfo.PipeId, AUTO_CLEAR_STALL, 1, &acs);
		}
		return true;
	}else{
		MessageBox(NULL, _T("WinUsb_QueryInterfaceSettings()Ɏs܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
		return false;
	}
}

void CUUSBB_WinUsb::Close(void)
{
	if(this->winUSBHandle)
		ptrWinUsb_Free(this->winUSBHandle);
	if(this->deviceHandle)
		CloseHandle(this->deviceHandle);

	this->winUSBHandle = this->deviceHandle = NULL;
}

bool CUUSBB_WinUsb::IsOpened(void)
{
	if(this->winUSBHandle)
		return true;
	return false;
}

bool CUUSBB_WinUsb::ReadPipe(BYTE pipeId, BYTE* buffer, DWORD bufferSize, DWORD *transferredSize)
{
	if(!this->loaded)
		return false;
	return ptrWinUsb_ReadPipe(this->winUSBHandle, pipeId, buffer, bufferSize, transferredSize, NULL)!=0;
}

bool CUUSBB_WinUsb::WritePipe(BYTE pipeId, BYTE* buffer, DWORD bufferSize, DWORD *transferredSize)
{
	if(!this->loaded)
		return false;
	return ptrWinUsb_WritePipe(this->winUSBHandle, pipeId, buffer, bufferSize, transferredSize, NULL)!=0;
}

CUUSBB_WinUsb::CUUSBB_WinUsb():winUSBHandle(NULL),deviceHandle(NULL),loaded(false)
{
	this->dllHandle = LoadLibrary(_T("winusb.dll"));
	if(this->dllHandle == NULL)
	{
		MessageBox(NULL, _T("WinUsb.dll̃[hɎs܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
		return;
	}

	ptrWinUsb_Initialize = (PWINUSB_INITIALIZE)GetProcAddress(this->dllHandle, "WinUsb_Initialize");
	ptrWinUsb_Free = (PWINUSB_FREE)GetProcAddress(this->dllHandle, "WinUsb_Free");
	ptrWinUsb_QueryInterfaceSettings = (PWINUSB_QUERYINTERFACESETTINGS)GetProcAddress(this->dllHandle, "WinUsb_QueryInterfaceSettings");
	ptrWinUsb_QueryPipe = (PWINUSB_QUERYPIPE)GetProcAddress(this->dllHandle, "WinUsb_QueryPipe");
	ptrWinUsb_SetPipePolicy = (PWINUSB_SETPIPEPOLICY)GetProcAddress(this->dllHandle, "WinUsb_SetPipePolicy");
	ptrWinUsb_ReadPipe = (PWINUSB_READPIPE)GetProcAddress(this->dllHandle, "WinUsb_ReadPipe");
	ptrWinUsb_WritePipe = (PWINUSB_WRITEPIPE)GetProcAddress(this->dllHandle, "WinUsb_WritePipe");

	this->loaded = true;
}

CUUSBB_WinUsb::~CUUSBB_WinUsb()
{
	FreeLibrary(this->dllHandle);
}

//internal methods
HANDLE CUUSBB_WinUsb::OpenDevice(DWORD index, const GUID &guid)
{
	HANDLE hDev = NULL;
	TCHAR devicePath[MAX_PATH];

	if( GetDevicePath(guid, index, devicePath, sizeof(devicePath)) )
	{
		hDev = CreateFile(devicePath, GENERIC_WRITE | GENERIC_READ, 0,
						NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED, NULL);
		if(hDev)
			return hDev;
	}

	return NULL;
}

BOOL CUUSBB_WinUsb::GetDevicePath(const GUID &InterfaceGuid, DWORD dwIndex, PTCHAR DevicePath, size_t BufLen)
{
	BOOL bResult = FALSE;
	HDEVINFO deviceInfo;
	SP_DEVICE_INTERFACE_DATA interfaceData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA detailData = NULL;
	DWORD length;
	DWORD requiredLength=0;
	HRESULT hr;

	deviceInfo = SetupDiGetClassDevs(&InterfaceGuid,  NULL, NULL, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
	if(!deviceInfo)
	{
		return FALSE;
	}

	interfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);

	bResult = SetupDiEnumDeviceInterfaces(deviceInfo, NULL, &InterfaceGuid, dwIndex, &interfaceData);
	if(!bResult)
	{
		SetupDiDestroyDeviceInfoList(deviceInfo);
		return FALSE;
	}

	SetupDiGetDeviceInterfaceDetail(deviceInfo,	&interfaceData, NULL, 0, &requiredLength, NULL);

	detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA) LocalAlloc(LMEM_FIXED, requiredLength);

	if(NULL == detailData)
	{
		SetupDiDestroyDeviceInfoList(deviceInfo);
		return FALSE;
	}

	detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
	length = requiredLength;

	bResult = SetupDiGetDeviceInterfaceDetail(deviceInfo, &interfaceData, detailData, length, &requiredLength, NULL);
	if(!bResult)
	{
		SetupDiDestroyDeviceInfoList(deviceInfo);
		LocalFree(detailData);
		return FALSE;
	}

	hr = StringCchCopy(DevicePath, BufLen, detailData->DevicePath);
	if(FAILED(hr))
	{
		SetupDiDestroyDeviceInfoList(deviceInfo);
		LocalFree(detailData);
		return FALSE;
	}

	LocalFree(detailData);

	return bResult;
}
